@@ -1,7 +1,7 @@
 use swc_atoms::js_word;
 use swc_common::{util::take::Take, Spanned, SyntaxContext};
 use swc_ecma_ast::*;
-use swc_ecma_utils::{ident::IdentLike, undefined, ExprExt, IsEmpty, Value};
+use swc_ecma_utils::{undefined, ExprExt, IsEmpty, Value};
 use swc_ecma_visit::VisitMutWith;
 
 use super::Pure;
@@ -112,7 +112,7 @@ impl Pure<'_> {
         let has_spread = call.args.iter().any(|arg| arg.spread.is_some());
 
         for arg in &call.args {
-            if arg.expr.may_have_side_effects() {
+            if arg.expr.may_have_side_effects(&self.expr_ctx) {
                 return;
             }
         }
@@ -128,7 +128,7 @@ impl Pure<'_> {
             prop: MemberProp::Ident(method_name),
         }) = callee
         {
-            if obj.may_have_side_effects() {
+            if obj.may_have_side_effects(&self.expr_ctx) {
                 return;
             }
 
@@ -161,7 +161,9 @@ impl Pure<'_> {
                         *e = *obj.take();
                     }
                     1 => {
-                        if let Value::Known(start) = call.args[0].expr.as_number() {
+                        if let Value::Known(start) =
+                            call.args[0].expr.as_pure_number(&self.expr_ctx)
+                        {
                             let start = start.floor() as usize;
 
                             self.changed = true;
@@ -181,8 +183,8 @@ impl Pure<'_> {
                         }
                     }
                     _ => {
-                        let start = call.args[0].expr.as_number();
-                        let end = call.args[1].expr.as_number();
+                        let start = call.args[0].expr.as_pure_number(&self.expr_ctx);
+                        let end = call.args[1].expr.as_pure_number(&self.expr_ctx);
                         if let Value::Known(start) = start {
                             let start = start.floor() as usize;
 
@@ -243,7 +245,7 @@ impl Pure<'_> {
         let has_spread = call.args.iter().any(|arg| arg.spread.is_some());
 
         for arg in &call.args {
-            if arg.expr.may_have_side_effects() {
+            if arg.expr.may_have_side_effects(&self.expr_ctx) {
                 return;
             }
         }
@@ -259,7 +261,7 @@ impl Pure<'_> {
             ..
         }) = callee
         {
-            if obj.may_have_side_effects() {
+            if obj.may_have_side_effects(&self.expr_ctx) {
                 return;
             }
 
@@ -360,12 +362,15 @@ impl Pure<'_> {
             _ => return,
         };
 
-        if args.iter().any(|arg| arg.expr.may_have_side_effects()) {
+        if args
+            .iter()
+            .any(|arg| arg.expr.may_have_side_effects(&self.expr_ctx))
+        {
             return;
         }
 
         if &*method.sym == "toFixed" {
-            if let Some(precision) = eval_as_number(&args[0].expr) {
+            if let Some(precision) = eval_as_number(&self.expr_ctx, &args[0].expr) {
                 let precision = precision.floor() as usize;
                 let value = num_to_fixed(num.value, precision + 1);
 
@@ -395,7 +400,7 @@ impl Pure<'_> {
         match &mut opt.base {
             OptChainBase::Member(MemberExpr { span, obj, .. }) => {
                 //
-                if is_pure_undefined_or_null(obj) {
+                if is_pure_undefined_or_null(&self.expr_ctx, obj) {
                     self.changed = true;
                     report_change!(
                         "evaluate: Reduced an optional chaining operation because object is \
@@ -407,7 +412,7 @@ impl Pure<'_> {
             }
 
             OptChainBase::Call(OptCall { span, callee, .. }) => {
-                if is_pure_undefined_or_null(callee) {
+                if is_pure_undefined_or_null(&self.expr_ctx, callee) {
                     self.changed = true;
                     report_change!(
                         "evaluate: Reduced a call expression with optional chaining operation \
@@ -435,7 +440,7 @@ impl Pure<'_> {
         };
 
         if bin_expr.op == op!("||") {
-            if let Value::Known(v) = bin_expr.right.as_pure_bool() {
+            if let Value::Known(v) = bin_expr.right.as_pure_bool(&self.expr_ctx) {
                 // foo || 1 => foo, 1
                 if v {
                     self.changed = true;
@@ -456,7 +461,7 @@ impl Pure<'_> {
             }
 
             // 1 || foo => foo
-            if let Value::Known(true) = bin_expr.left.as_pure_bool() {
+            if let Value::Known(true) = bin_expr.left.as_pure_bool(&self.expr_ctx) {
                 self.changed = true;
                 report_change!("evaluate: `true || foo` => `foo`");
 
@@ -465,7 +470,7 @@ impl Pure<'_> {
         } else {
             debug_assert_eq!(bin_expr.op, op!("&&"));
 
-            if let Value::Known(v) = bin_expr.right.as_pure_bool() {
+            if let Value::Known(v) = bin_expr.right.as_pure_bool(&self.expr_ctx) {
                 if v {
                     self.changed = true;
                     report_change!("evaluate: `foo && true` => `foo` (bool ctx)");
@@ -484,7 +489,7 @@ impl Pure<'_> {
                 return;
             }
 
-            if let Value::Known(true) = bin_expr.left.as_pure_bool() {
+            if let Value::Known(true) = bin_expr.left.as_pure_bool(&self.expr_ctx) {
                 self.changed = true;
                 report_change!("evaluate: `true && foo` => `foo`");
 
