@@ -26,7 +26,7 @@ impl Pure<'_> {
                 });
             }
             Stmt::DoWhile(stmt) => {
-                let val = stmt.test.as_pure_bool();
+                let val = stmt.test.as_pure_bool(&self.expr_ctx);
                 if let Value::Known(true) = val {
                     self.changed = true;
                     report_change!("loops: Converting an always-true do-while loop to a for loop");
