@@ -3,7 +3,7 @@ use swc_common::{
     Mark, Span, SyntaxContext,
 };
 use swc_ecma_ast::*;
-use swc_ecma_utils::{find_ids, ident::IdentLike, Id};
+use swc_ecma_utils::find_pat_ids;
 use swc_ecma_visit::{
     noop_visit_mut_type, noop_visit_type, Visit, VisitMut, VisitMutWith, VisitWith,
 };
@@ -232,7 +232,7 @@ impl Visit for TopLevelBindingCollector {
 
     fn visit_var_decl(&mut self, v: &VarDecl) {
         v.visit_children_with(self);
-        let ids: Vec<Id> = find_ids(&v.decls);
+        let ids: Vec<Id> = find_pat_ids(&v.decls);
 
         for id in ids {
             self.add(id)
